﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class TerrainChunk : MonoBehaviour
{
    [Header("Autogenerated values")]
    [Tooltip("Current grid position of the chunker")]
    public Vector2Int gridPosition;

    public Vector2 crawlOffset;

    public TerrainMaster master;

    public float TerrainSize => terrainWidth;

    protected Terrain terrain;

    protected TerrainData data;

    protected int heightResolution = 513;

    protected float terrainWidth = 1000;

    protected float heightScale;

    protected float[,] biomeData;

    [Header("Slave chunks")]
    public BiomeChunk biomeChunk;

    public ShapeChunk shapeChunk;

    private List<SlaveChunk> slaves;

    public enum Phase
    {
        Idle,
        WaitingForBiome,
        WaitingForShape,
        WaitingForTexture,
        WaitingForTrees
    }

    public Phase currentPhase;

    public void Initialize(TerrainMaster master, Vector2Int gridPosition,
        Vector2 crawlOffset)
    {
        this.master = master;
        this.gridPosition = gridPosition;
        this.crawlOffset = crawlOffset;

        gameObject.name = $"Chunker {gridPosition}";

        data = Instantiate(Resources.Load<TerrainData>("Template"));

        heightScale = data.size.y / master.terrainHeight;
        data.size = new(data.size.x, master.terrainHeight, data.size.z);

        heightResolution = data.heightmapResolution;
        terrainWidth = data.size.x;

        slaves = new();
        slaves.Add(biomeChunk);

        UpdateChunk(gridPosition, true);
    }

    public void UpdateChunk(Vector2Int newGridPosition, bool forceUpdate = false)
    {
        if (newGridPosition != gridPosition || forceUpdate)
        {
            gridPosition = newGridPosition;

            // Now we update the terrain
            UpdateTerrain();
        }
    }

    private void UpdateTerrain()
    {
        print($"Updating {gameObject.name} terrain.");

        biomeChunk.CreateGenerator();
        currentPhase = Phase.WaitingForBiome;
    }

    private void ApplyTerrain(float[,] heights)
    {
        print($"Apply {gameObject.name} terrain with mode {master.mode}.");

        // Calculate position of game object
        Vector2 flatPosition = (Vector2)gridPosition * terrainWidth;
        Vector3 worldPosition = new(flatPosition.x, 0, flatPosition.y);

        // Set heights of terrain data.
        master.shapeApplier.ApplyHeights(transform, terrain, data,
           worldPosition, heights);
    }

    private void Update()
    {
        if (currentPhase == Phase.WaitingForBiome && biomeChunk.IsDone() && !biomeChunk.IsCancelled())
        {
            biomeData = biomeChunk.GetBiomeData();
            currentPhase = Phase.WaitingForTexture;
        }
    }

    private void OnDestroy()
    {
        foreach (var slave in slaves)
        {
            slave.CleanUp();
        }
    }
}