﻿using System.Collections;
using System.Linq;
using UnityEngine;
using UnityEngine.UI;
using TMPro;

public class FadeInOut : MonoBehaviour
{
    [Header("User settings")]
    public AnimationCurve fadeCurve;

    public bool startHidden;

    [Header("Autogenerated values")]
    public Image[] imageTargets;

    public TMP_Text[] textTargets;

    public MeshRenderer[] meshTargets;

    public enum State
    {
        FadedOut, FadedIn,
        FadingOut, FadingIn
    }
    public State FadeState { get; private set; }

    private float t;
    private int direction;
    private float duration;
    private Transform previousParent;

    private void Start()
    {
        imageTargets = GetComponentsInChildren<Image>(true);
        textTargets = GetComponentsInChildren<TMP_Text>(true);
        meshTargets = GetComponentsInChildren<MeshRenderer>(true);

        SetTargetsAlpha(startHidden ? 0 : 1);
        FadeState = startHidden ? State.FadedOut : State.FadedIn;

        duration = fadeCurve.keys.Max(k => k.time);
    }

    private void Update()
    {
        if (direction == 1)
        {
            if (t < duration)
            {
                t += Time.deltaTime;
                SetTargetsAlpha(fadeCurve.Evaluate(t));
            }
            else
            {
                t = 0;
                direction = 0;
                FadeState = State.FadedIn;
            }
        }
        else if (direction == -1)
        {
            if (t < duration)
            {
                t += Time.deltaTime;
                SetTargetsAlpha(fadeCurve.Evaluate(duration - t));
            }
            else
            {
                t = 0;
                direction = 0;
                FadeState = State.FadedOut;

                if (previousParent)
                {
                    transform.parent = previousParent;
                    previousParent = null;
                }
            }
        }
    }

    private void SetTargetsAlpha(float alpha)
    {
        foreach (var target in imageTargets)
        {
            Color targetColor = target.color;
            targetColor.a = alpha;
            target.color = targetColor;
        }

        foreach (var target in textTargets)
        {
            Color targetColor = target.color;
            targetColor.a = alpha;
            target.color = targetColor;
        }

        foreach (var target in meshTargets)
        {
            Color targetColor = target.material.color;
            targetColor.a = alpha;
            target.material.color = targetColor;
        }
    }

    public void FadeIn()
    {
        SetTargetsAlpha(0);

        direction = 1;
        t = 0;
        FadeState = State.FadingIn;
    }

    public void FadeOut(bool asOrphan = false)
    {
        SetTargetsAlpha(1);

        direction = -1;
        t = 0;
        FadeState = State.FadingOut;

        if (asOrphan)
        {
            previousParent = transform.parent;
            transform.parent = null;
        }
    }
}